package org.jbosson.plugins.jbossesb;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.mc4j.ems.connection.bean.EmsBean;
import org.rhq.core.domain.configuration.Configuration;
import org.rhq.core.domain.configuration.PropertySimple;
import org.rhq.core.domain.content.PackageDetailsKey;
import org.rhq.core.domain.content.PackageType;
import org.rhq.core.domain.content.transfer.ContentResponseResult;
import org.rhq.core.domain.content.transfer.DeployIndividualPackageResponse;
import org.rhq.core.domain.content.transfer.DeployPackageStep;
import org.rhq.core.domain.content.transfer.DeployPackagesResponse;
import org.rhq.core.domain.content.transfer.RemovePackagesResponse;
import org.rhq.core.domain.content.transfer.ResourcePackageDetails;
import org.rhq.core.domain.measurement.AvailabilityType;
import org.rhq.core.domain.resource.CreateResourceStatus;
import org.rhq.core.pluginapi.content.ContentContext;
import org.rhq.core.pluginapi.content.ContentFacet;
import org.rhq.core.pluginapi.content.ContentServices;
import org.rhq.core.pluginapi.event.EventContext;
import org.rhq.core.pluginapi.inventory.CreateResourceReport;
import org.rhq.core.pluginapi.inventory.DeleteResourceFacet;
import org.rhq.core.pluginapi.inventory.ResourceContext;
import org.rhq.core.pluginapi.operation.OperationContext;
import org.rhq.core.pluginapi.operation.OperationFacet;
import org.rhq.plugins.jbossas.JBossASServerComponent;
import org.rhq.plugins.jbossas.util.FileContentDelegate;
import org.rhq.plugins.jmx.JMXComponent;
import org.rhq.plugins.jmx.MBeanResourceComponent;
//import org.rhq.core.util.file.FileUtil;

/**
 * Component for ESB package deployment.
 *
 * @author Tom Cunningham
 * @param <T>
 */
public class DeploymentComponent extends MBeanResourceComponent<JMXComponent> implements 
	OperationFacet, DeleteResourceFacet, ContentFacet {
    public static final String JBOSS_WEB_NAME = "jbossWebName";
    private EmsBean jbossWebMBean;
    private File configPath;

    private ResourceContext resourceContext;
    private ContentContext contentContext;
    private OperationContext operationContext;
    private EventContext eventContext;
    
    // The following constants reference the exact name of the package types as defined in the plugin descriptor
    private static final String PACKAGE_TYPE_PATCH = "cumulativePatch";
    private static final String PACKAGE_TYPE_LIBRARY = "library";


    private static final String RESOURCE_TYPE_ESB = "JBoss ESB Deployments";

    public File getConfigurationPath() {
        return this.configPath;
    }
        
    public CreateResourceReport createResource(CreateResourceReport report) {
        String resourceTypeName = report.getResourceType().getName();

        if (resourceTypeName.equals(RESOURCE_TYPE_ESB)) {
        	esbCreate(report, resourceTypeName);
        } else {
            throw new UnsupportedOperationException("Unknown Resource type: " + resourceTypeName);
        }

        return report;
    }
    
    private void esbCreate(CreateResourceReport report, String resourceTypeName) {
        ResourcePackageDetails details = report.getPackageDetails();
        PackageDetailsKey key = details.getKey();
        String archiveName = key.getName();

        try {
            // First check to see if the file name has the correct extension. Reject if the user attempts to
            // deploy a WAR file with a bad extension.
            String expectedExtension;
            if (resourceTypeName.equals(RESOURCE_TYPE_ESB)) {
                expectedExtension = "esb";
            } else {
            	expectedExtension = "";
            }
            
            int lastPeriod = archiveName.lastIndexOf(".");
            String extension = archiveName.substring(lastPeriod + 1);
            if (lastPeriod == -1 || !expectedExtension.equals(extension)) {
                report.setStatus(CreateResourceStatus.FAILURE);
                report.setErrorMessage("Incorrect extension specified on filename [" + archiveName + "]. Expected ["
                    + expectedExtension + "]");
                return;
            }

            Configuration deployTimeConfiguration = details.getDeploymentTimeConfiguration();
            String deployDirectory = deployTimeConfiguration.getSimple("deployDirectory").getStringValue();

            // Verify the user did not enter a path that represents a security issue:
            // - No absolute directories; must be relative to the configuration path
            // - Cannot contain parent directory references
            File testPath = new File(deployDirectory);

            if (testPath.isAbsolute()) {
                throw new RuntimeException("Path to deploy (deployDirectory) must be a relative path. Path specified: "
                    + deployDirectory);
            }

            if (deployDirectory.contains("..")) {
                throw new RuntimeException(
                    "Path to deploy (deployDirectory) may not reference the parent directory. Path specified: "
                        + deployDirectory);
            }

            // Perform the deployment
            FileContentDelegate deployer = new FileContentDelegate(new File(getConfigurationPath() + File.separator
                + deployDirectory), "", details.getPackageTypeName());

            PropertySimple zipProperty = deployTimeConfiguration.getSimple("deployZipped");

            if (zipProperty != null && zipProperty.getBooleanValue() != null) {
                boolean zip = zipProperty.getBooleanValue();

                File tempDir = resourceContext.getTemporaryDirectory();
                File tempFile = new File(tempDir.getAbsolutePath(), "esb.bin");
                OutputStream osForTempDir = new BufferedOutputStream(new FileOutputStream(tempFile));

                ContentServices contentServices = contentContext.getContentServices();
                contentServices
                    .downloadPackageBitsForChildResource(contentContext, resourceTypeName, key, osForTempDir);

                osForTempDir.close();

                // check for content 
                boolean valid = isOfType(tempFile, resourceTypeName);
                if (!valid) {
                    report.setStatus(CreateResourceStatus.FAILURE);
                    report.setErrorMessage("Expected a " + resourceTypeName
                        + " file, but its format/content did not match");
                    return;
                }

                InputStream isForTempDir = new BufferedInputStream(new FileInputStream(tempFile));
                deployer.createContent(details, isForTempDir, !zip, true);

                // Resource key should match the following:      
                // EAR: jboss.management.local:J2EEServer=Local,j2eeType=J2EEApplication,name=rhq.ear      
                // WAR: jboss.management.local:J2EEApplication=null,J2EEServer=Local,j2eeType=WebModule,name=embedded-console.war

                String resourceKey;
                if (resourceTypeName.equals(RESOURCE_TYPE_ESB)) {
                    resourceKey = "jboss.esb:deployment="
                        + archiveName;
                } 

                report.setResourceName(archiveName);
                //report.setResourceKey(resourceKey);
                report.setStatus(CreateResourceStatus.SUCCESS);
                sleepAfterConfigXmlUpdate();
            } else {
                report.setStatus(CreateResourceStatus.FAILURE);
                report.setErrorMessage("Zipped property is required");
            }
        } catch (Throwable t) {
            log.error("Error deploying application for report: " + report, t);
            report.setException(t);
            report.setStatus(CreateResourceStatus.FAILURE);
        }
    }

    /**
     * Check to see if the passed file is actually in jar format and contains a 
     * <ul>
     * <li>WEB-INF/web.xml for .war </li>
     * <li>META-INF/application.xml for .ear</li>
     * <li>META-INF/jboss.service.xml for .sar</li>
     * </ul>
     * @param file File to check
     * @param type Type to match - see RESOURCE_TYPE_SAR, RESOURCE_TYPE_WAR and RESOURCE_TYPE_EAR
     * @return true is the file is in jar format and matches the type
     */
    private boolean isOfType(File file, String type) {
        JarFile jfile = null;
        try {
            jfile = new JarFile(file);
            JarEntry entry;
            if (RESOURCE_TYPE_ESB.equals(type))
                entry = jfile.getJarEntry("META-INF/jboss-esb.xml");
            else {
                entry = null; // unknown type
                log.warn("isOfType: " + type + " is unknown - not a valid file");
            }

            if (entry != null)
                return true;

            return false;
        } catch (Exception e) {
            log.info(e.getMessage());
            return false;
        } finally {
            if (jfile != null)
                try {
                    jfile.close();
                } catch (IOException e) {
                    log.info("Exception when trying to close the war file: " + e.getMessage());
                }
        }
    }
    
    private void sleepAfterConfigXmlUpdate() {
        // JBNADM-1984 - The contract with this method is that the newly created managed resource should be discoverable.
        //               Wait here so JBoss can recognize that the new managed resource has been created.
        try {
            Thread.sleep(5000L);
        } catch (InterruptedException e) {
            log.info("Sleep after Resource create interrupted", e);
        }
    }
    
    @Override
	public AvailabilityType getAvailability() {
        //        JBossASTomcatServerComponent parentTomcatComponent = (JBossASTomcatServerComponent) super.resourceContext
        //            .getParentResourceComponent();
        //        EmsConnection connection = parentTomcatComponent.getEmsConnection();
        boolean isreg = this.getEmsBean().isRegistered();
        return isreg ? AvailabilityType.UP : AvailabilityType.DOWN;
    }

	public void deleteResource() throws Exception {
        Configuration pluginConfiguration = super.getResourceContext().getPluginConfiguration();
        String fullFileName = pluginConfiguration.getSimple("deployment").getStringValue();
        
        ESBComponent jbossASComponent = (ESBComponent) super.getResourceContext().getParentResourceComponent();
        //File deploymentFile = jbossASComponent.getDeploymentFilePath(super.resourceContext.getResourceKey());

        File file = new File(jbossASComponent.getConfigurationPath() + "/deploy/" + fullFileName);

        if (!file.exists()) {
            throw new Exception("Cannot find application file to delete: " + fullFileName);
        }

        if (file.isDirectory()) {
            //FileUtils.deleteDirectoryContents(file.listFiles());
            log.info("Attempting to delete file (not supported)");
        }

        boolean result = file.delete();

        if (!result) {
            throw new Exception("File delete call returned unsuccessful with no further detail");
        }
	}

	public DeployPackagesResponse deployPackages(
			Set<ResourcePackageDetails> packages,
			ContentServices contentServices) {
        ContentResponseResult overallResult = ContentResponseResult.SUCCESS;
        List<DeployIndividualPackageResponse> individualResponses = new ArrayList<DeployIndividualPackageResponse>(
            packages.size());

        for (ResourcePackageDetails pkg : packages) {
            log.info("Attempting to deploy package: " + pkg);

            String packageTypeName = pkg.getPackageTypeName();
            if (packageTypeName.equals(PACKAGE_TYPE_LIBRARY)) {
                throw new UnsupportedOperationException("Deployment of new libraries is not supported by the plugin.");
            }
        }

        DeployPackagesResponse response = new DeployPackagesResponse(overallResult);
        response.getPackageResponses().addAll(individualResponses);

        return response;
	}

	public Set<ResourcePackageDetails> discoverDeployedPackages(PackageType type) {
		return null;
	}

	public List<DeployPackageStep> generateInstallationSteps(
			ResourcePackageDetails packageDetails) {
		return null;
	}

	public RemovePackagesResponse removePackages(
			Set<ResourcePackageDetails> packages) {
		return null;
	}

	public InputStream retrievePackageBits(ResourcePackageDetails packageDetails) {
		return null;
	}

}
