package org.jbosson.plugins.jbossesb;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.rhq.core.domain.configuration.Configuration;
import org.rhq.core.domain.configuration.PropertySimple;
import org.rhq.core.domain.content.PackageDetailsKey;
import org.rhq.core.domain.content.PackageType;
import org.rhq.core.domain.content.transfer.ContentResponseResult;
import org.rhq.core.domain.content.transfer.DeployIndividualPackageResponse;
import org.rhq.core.domain.content.transfer.DeployPackageStep;
import org.rhq.core.domain.content.transfer.DeployPackagesResponse;
import org.rhq.core.domain.content.transfer.RemovePackagesResponse;
import org.rhq.core.domain.content.transfer.ResourcePackageDetails;
import org.rhq.core.domain.measurement.AvailabilityType;
import org.rhq.core.domain.resource.CreateResourceStatus;
import org.rhq.core.pluginapi.content.ContentContext;
import org.rhq.core.pluginapi.content.ContentFacet;
import org.rhq.core.pluginapi.content.ContentServices;
import org.rhq.core.pluginapi.inventory.CreateChildResourceFacet;
import org.rhq.core.pluginapi.inventory.CreateResourceReport;
import org.rhq.core.pluginapi.inventory.InvalidPluginConfigurationException;
import org.rhq.core.pluginapi.inventory.ResourceContext;
import org.rhq.core.pluginapi.operation.OperationContext;
import org.rhq.core.pluginapi.util.ResponseTimeLogParser;
import org.rhq.plugins.jbossas.JBossASServerComponent;
import org.rhq.plugins.jbossas.util.FileContentDelegate;
import org.rhq.plugins.jmx.MBeanResourceComponent;

public class ESBComponent extends MBeanResourceComponent implements CreateChildResourceFacet, ContentFacet {
   private static final String RESOURCE_TYPE_ESB = "JBoss ESB Deployment";

   private String configSet;
   private ResponseTimeLogParser logParser;
   private File configPath;

   private ContentContext contentContext;
   private OperationContext operationContext;
   public static final String CONTEXT_ROOT_CONFIG_PROP = "contextRoot";
   // The following constants reference the exact name of the package types as defined in the plugin descriptor
   private static final String PACKAGE_TYPE_PATCH = "cumulativePatch";
   private static final String PACKAGE_TYPE_LIBRARY = "library";

   public static final String CONFIGURATION_PATH_CONFIG_PROP = "configurationPath";
   public static final String SCRIPT_PREFIX_CONFIG_PROP = "scriptPrefix";
   public static final String CONFIGURATION_SET_CONFIG_PROP = "configurationSet";

   public static final String JBOSS_HOME_DIR_CONFIG_PROP = "jbossHomeDir";

   
   @Override
   public void start(ResourceContext context) {
	   super.start(context);
	   Configuration pluginConfig = context.getPluginConfiguration();

	   this.configPath = getConfigurationPath();
	   if (!this.configPath.exists()) {
		   throw new InvalidPluginConfigurationException("Configuration path '" + configPath + "' does not exist.");
	   }
	   this.configSet = pluginConfig.getSimpleValue(CONFIGURATION_SET_CONFIG_PROP, this.configPath.getName());
   }
   
   public File getConfigurationPath() {
	   JBossASServerComponent jass = (JBossASServerComponent) this.getResourceContext().getParentResourceComponent();
	   return jass.getConfigurationPath();
   }
     
   @Override
   public AvailabilityType getAvailability() {
	   AvailabilityType av = super.getAvailability();
	   return av;
   }
      
   private void esbCreate(CreateResourceReport report, String resourceTypeName) {
        ResourcePackageDetails details = report.getPackageDetails();
        PackageDetailsKey key = details.getKey();
        String archiveName = key.getName();

        try {
            // First check to see if the file name has the correct extension. Reject if the user attempts to
            // deploy a WAR file with a bad extension.
            String expectedExtension;
            if (resourceTypeName.equals(RESOURCE_TYPE_ESB)) {
                expectedExtension = "esb";
            } else {
                expectedExtension = "";
            }
	        
            int lastPeriod = archiveName.lastIndexOf(".");
	        String extension = archiveName.substring(lastPeriod + 1);
	        if (lastPeriod == -1 || !expectedExtension.equals(extension)) {
	        	report.setStatus(CreateResourceStatus.FAILURE);
	            report.setErrorMessage("Incorrect extension specified on filename [" + archiveName + "]. Expected ["
	            	+ expectedExtension + "]");
	            return;
	        }

            Configuration deployTimeConfiguration = details.getDeploymentTimeConfiguration();
            String deployDirectory = deployTimeConfiguration.getSimple("deployDirectory").getStringValue();

            // Verify the user did not enter a path that represents a security issue:
            // - No absolute directories; must be relative to the configuration path
            // - Cannot contain parent directory references
            File testPath = new File(deployDirectory);

            if (testPath.isAbsolute()) {
            	throw new RuntimeException("Path to deploy (deployDirectory) must be a relative path. Path specified: "
	                    + deployDirectory);
            }
            
            if (deployDirectory.contains("..")) {
	        	throw new RuntimeException(
	            	"Path to deploy (deployDirectory) may not reference the parent directory. Path specified: "
	                	+ deployDirectory);
	        }

	        // Perform the deployment
            FileContentDelegate deployer = new FileContentDelegate(new File(getConfigurationPath() + File.separator
            	+ deployDirectory), "", details.getPackageTypeName());

            PropertySimple zipProperty = deployTimeConfiguration.getSimple("deployZipped");

	        if (zipProperty != null && zipProperty.getBooleanValue() != null) {
	        	boolean zip = zipProperty.getBooleanValue();

	            File tempDir = this.getResourceContext().getTemporaryDirectory();
	            File tempFile = new File(tempDir.getAbsolutePath(), "esb.bin");
	            OutputStream osForTempDir = new BufferedOutputStream(new FileOutputStream(tempFile));

	            contentContext = this.getResourceContext().getContentContext();
	            ContentServices contentServices = contentContext.getContentServices();
	            contentServices
	                    .downloadPackageBitsForChildResource(contentContext, resourceTypeName, key, osForTempDir);

	            osForTempDir.close();

	            InputStream isForTempDir = new BufferedInputStream(new FileInputStream(tempFile));
	            deployer.createContent(details, isForTempDir, !zip, true);

                // Resource key should match the following:      
                // ESB: jboss.esb:deployment=jbossesb.esb

                String resourceKey;
                resourceKey = "jboss.esb:deployment=" + archiveName;

                report.setResourceName(archiveName);
                report.setResourceKey(resourceKey);
                report.setStatus(CreateResourceStatus.SUCCESS);
            } else {
                report.setStatus(CreateResourceStatus.FAILURE);
                report.setErrorMessage("Zipped property is required");
            }
        } catch (Throwable t) {
            log.error("Error deploying application for report: " + report, t);
            report.setException(t);
            report.setStatus(CreateResourceStatus.FAILURE);
        }
    }
	
	public CreateResourceReport createResource(CreateResourceReport report) {
        String resourceTypeName = report.getResourceType().getName();

        if (resourceTypeName.equals(RESOURCE_TYPE_ESB)) {
            esbCreate(report, resourceTypeName);
        } else {
            throw new UnsupportedOperationException("Unknown Resource type: " + resourceTypeName);
        }

        // JBNADM-1984 - The contract with this method is that the newly created managed resource should be discoverable.
        //               Wait here so JBoss can recognize that the new managed resource has been created.
        try {
            Thread.sleep(5000L);
        } catch (InterruptedException e) {
            log.info("Sleep after datasource create interrupted", e);
        }

        return report;
    }

	public DeployPackagesResponse deployPackages(
			Set<ResourcePackageDetails> packages,
			ContentServices contentServices) {
        ContentResponseResult overallResult = ContentResponseResult.SUCCESS;
        List<DeployIndividualPackageResponse> individualResponses = new ArrayList<DeployIndividualPackageResponse>(
            packages.size());

        for (ResourcePackageDetails pkg : packages) {
            log.info("Attempting to deploy package: " + pkg);

            String packageTypeName = pkg.getPackageTypeName();
            if (packageTypeName.equals(PACKAGE_TYPE_LIBRARY)) {
                throw new UnsupportedOperationException("Deployment of new libraries is not supported by the plugin.");
            }
        }

        DeployPackagesResponse response = new DeployPackagesResponse(overallResult);
        response.getPackageResponses().addAll(individualResponses);

        return response;
	}

	public Set<ResourcePackageDetails> discoverDeployedPackages(PackageType type) {
		return null;
	}

	public List<DeployPackageStep> generateInstallationSteps(
			ResourcePackageDetails packageDetails) {
		return null;
	}

	public RemovePackagesResponse removePackages(
			Set<ResourcePackageDetails> packages) {
        throw new UnsupportedOperationException();
	}

	public InputStream retrievePackageBits(ResourcePackageDetails packageDetails) {
        throw new UnsupportedOperationException();
	}
	
	
}

